Package: ExperimentHub
Type: Package
Title: Client to access ExperimentHub resources
Version: 3.2.0
Authors@R: 
    c(person("Bioconductor Package", "Maintainer", email="maintainer@bioconductor.org",
             role="cre"),
      person("Martin", "Morgan", role="aut"),
      person("Marc", "Carlson", role="ctb"),
      person("Dan", "Tenenbaum", role="ctb"),
      person("Sonali", "Arora", role="ctb"),
      person("Valerie", "Oberchain", role="ctb"),
      person("Kayla", "Morrell", role="ctb"),
      person("Lori", "Shepherd", role="aut"))
Description: This package provides a client for the Bioconductor
        ExperimentHub web resource. ExperimentHub provides a central location
        where curated data from experiments, publications or training courses
        can be accessed. Each resource has associated metadata, tags and date
        of modification. The client creates and manages a local cache of files
        retrieved enabling quick and reproducible access.
License: Artistic-2.0
biocViews: Infrastructure, DataImport, GUI, ThirdPartyClient
Depends: methods, BiocGenerics (>= 0.15.10), AnnotationHub (>= 3.99.3),
        BiocFileCache (>= 2.99.3)
Imports: utils, S4Vectors, BiocManager, rappdirs
Suggests: knitr, BiocStyle, rmarkdown, HubPub, GenomicAlignments
Enhances: ExperimentHubData
URL: https://github.com/Bioconductor/ExperimentHub
BugReports: https://github.com/Bioconductor/ExperimentHub/issues
VignetteBuilder: knitr
RoxygenNote: 7.0.0
git_url: https://git.bioconductor.org/packages/ExperimentHub
git_branch: RELEASE_3_23
git_last_commit: 2b9e0cc
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:07:55 UTC; biocbuild
Author: Bioconductor Package Maintainer [cre],
  Martin Morgan [aut],
  Marc Carlson [ctb],
  Dan Tenenbaum [ctb],
  Sonali Arora [ctb],
  Valerie Oberchain [ctb],
  Kayla Morrell [ctb],
  Lori Shepherd [aut]
Maintainer: Bioconductor Package Maintainer <maintainer@bioconductor.org>
Built: R 4.6.0; ; 2026-04-29 04:42:27 UTC; unix
