import(methods)
import(BiocGenerics)
import(S4Vectors)
import(AnnotationHub)
importFrom(BiocManager, install)
import(BiocFileCache)
importFrom(tools, R_user_dir)
importFrom(rappdirs, user_cache_dir)

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Export S4 classes
###

exportClasses(ExperimentHub) 

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Export S4 methods for generics not defined in AnnotationHub 
###

exportMethods(
    cache, "[[", 
    listResources, loadResources, package
)

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Export S4 generics defined in AnnotationHub + export corresponding
### methods
###

### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
### Export non-generic functions
###

export(ExperimentHub, 
       getExperimentHubOption, setExperimentHubOption,
       createHubAccessors
)
